/**
 * 孵化器基础数据相关逻辑
 * 作者: 李某
 * 孵化器相关逻辑 包括孵化器的增删改查
 * 
 */
import { BaseParamUpdateConfig, OrganizationParamUpdateConfig } from "../../config/checkParamConfig";
import { findEnterpriseCountByFuHuaQiUscc, groupFindEnterprise } from "../../data/enterprise/enterprise";
import * as fuhuaqiData from "../../data/fuHuaQi/fuhuaqi";
import * as resultConfig from "../../config/ojbectResultKeyConfig";
import * as configEnum from "../../config/enum";
import { checkChange, extractData } from "../../util/piecemeal";
import { BizError } from "../../util/bizError";
import { ERRORENUM } from "../../config/errorEnum";
import { eccFormParam } from "../../util/verificationParam";
import { changeEnumValue } from "../../util/verificationEnum";


/**
 * 主界面基础数据
 * 小程序端  首页的基础数据
 * 评分数据还没有
 * @param uscc 孵化器统一信用代码
 * @returns {} "incubatedAcreage", "acreageTotal", "acreagePersonalUse", "name", "lv", "institutionalNature"
 */
export async function homeData(uscc:string) {
    let dataBaseInfo = await fuhuaqiData.findFuHuaQiByUSCC(uscc);

    let data:any = extractData(resultConfig.FuHuaQiHomeDataConfig, dataBaseInfo, false);

    data.enterpriseCount  = await findEnterpriseCountByFuHuaQiUscc(uscc);
    data.lv = changeEnumValue(configEnum.FUHUAQILV, dataBaseInfo.lv);
    data.institutionalNature = changeEnumValue(configEnum.INSTITUTIONALNATURE, dataBaseInfo.institutionalNature);

    return data;
}


/**
 * 孵化器详细数据
 * 小程序端  点击我的数据之后拉取的详细数据
 * @param uscc 孵化器统一信用代码
 * @returns data 基础数据字段
 */
export async function selectFuHuaQiBaseData(uscc:string) {
    let dataBaseInfo = await fuhuaqiData.findFuHuaQiByUSCC(uscc);

    let enterpriseTotal = await findEnterpriseCountByFuHuaQiUscc(uscc); //企业总数
    
    let data:any = extractData(resultConfig.FuHuaQiBaseDataConfig, dataBaseInfo, false);
    
    data.enterpriseTotal = enterpriseTotal;
    data.numOfEntities = enterpriseTotal - dataBaseInfo.virtualEnterpriseNum; //企业总数 - 虚拟企业数量

    return data;
}


/**
 * 孵化器机构信息数据数据
 * 小程序端  点击机构信息之后拉取的详细数据
 * @param uscc 孵化器统一信用代码
 * @returns data 基础数据字段
 */
export async function selectOrganizationInfo(uscc:string) {
    let dataBaseInfo = await fuhuaqiData.findFuHuaQiByUSCC(uscc);
    
    let data:any = extractData(resultConfig.OrganizationBaseDataConfig, dataBaseInfo, false);

    return data;
}


/**
 * 更新孵化器基础数据
 * 小程序端
 * @param uscc 孵化器统一信用代码
 * @param param 表单内容
 * @returns {isSuccess:true/false};
 */
export async function updateFuHuaQiBaseData(uscc:string, param) {
    let skipKeyList = [
        "virtualEnterpriseNum", "incubatedAcreage", "acreageTotal", "acreagePersonalUse", "hatchingGround"
    ];
    eccFormParam("更新孵化器基础数据", BaseParamUpdateConfig, param, skipKeyList);

    let baseDataInfo = await fuhuaqiData.findFuHuaQiByUSCC(uscc);
    for (let key in param) {
        if (key == "hatchingGround") {
            param[key].forEach (info => {
                let {personInChargeAdd, siteAcreage, leasePrice} = info;
                if ( typeof personInChargeAdd != 'string' && personInChargeAdd) throw new BizError(ERRORENUM.参数错误, `personInChargeAdd应是string型 而不是${typeof personInChargeAdd}`);
                if ( typeof siteAcreage != 'number' && siteAcreage) throw new BizError(ERRORENUM.参数错误, `siteAcreage应是number型 而不是${typeof personInChargeAdd}`);
                if ( typeof leasePrice != 'number' && leasePrice) throw new BizError(ERRORENUM.参数错误, `leasePrice应是number型 而不是${typeof personInChargeAdd}`);
            })
        }
    }
        
    /**这里无法判断数组里面的内容是否有变化 所以 hatchingGround 直接赋值  */
    let changeList = checkChange(param, baseDataInfo);
    for (let i = 0; i < changeList.length; i++) {
        let key = changeList[i];
        baseDataInfo[key] = param[key];
    }
    baseDataInfo["hatchingGround"] = param.hatchingGround;

    await baseDataInfo.save();

    return {isSuccess:true};
}


/**
 * 更新孵化器机构信息数据
 * 小程序端
 * @param uscc 孵化器统一信用代码
 * @param param 表单内容
 * @returns {isSuccess:true/false};
 */
export async function updateOrganizationData(uscc:string, param) {
    let skipKeyList = [
        "name","virtualEnterpriseNum", "hatchingGround", "operationModelDes"//todo  写到这里了 这里和甘明讨论好哪些是必填的
        ,"logonTime", "lv", "identificationTime", "industry", "institutionalNature", "foundingTeam", "liaison", "liaisonPhone", "personInCharge", "personInChargePhone",
    ];
    eccFormParam("更新孵化器基础数据", OrganizationParamUpdateConfig, param, skipKeyList);

    let baseDataInfo = await fuhuaqiData.findFuHuaQiByUSCC(uscc);
        
    /**这里无法判断数组里面的内容是否有变化 所以 hatchingGround 直接赋值  */
    let changeList = checkChange(param, baseDataInfo);
    for (let i = 0; i < changeList.length; i++) {
        let key = changeList[i];
        if (key == "operationName" || key == "uscc") continue;
        
        baseDataInfo[key] = param[key];
    }
    baseDataInfo["foundingTeam"] = param.foundingTeam;
    if (param.foundingTeam && param.foundingTeam.length > 0) {
        baseDataInfo.foundingTeamType = param.foundingTeam[0].type;
    } else baseDataInfo.foundingTeamType = 0;

    await baseDataInfo.save();

    return {isSuccess:true};
}


/**
 * 获取孵化器基本信息列表
 * 管理后台端
 * @param lv 孵化器等级
 * @param institutionalNature 机构性质
 * @param industry 领域
 * @param page 页数
 * @returns 
 */
export async function fuHuaQiBaseList(lv:number, institutionalNature:number, industry:number, operationModel:number, foundingTeam:number, page:number) {
    let selectParam:any = {};
    if (lv) selectParam.lv = lv;
    if (institutionalNature) selectParam.institutionalNature = institutionalNature;
    if (industry) selectParam.industry = industry;
    if (operationModel) selectParam.operationModel = operationModel;
    if (foundingTeam) selectParam.foundingTeamType = foundingTeam;

    let enterpriseMap = await groupFindEnterprise();//企业总数

    let dataBaseList = await fuhuaqiData.findFuHuaQiListByPage(selectParam, (page-1)*10 );

    let count = await fuhuaqiData.findFuHuaQiCount(selectParam);
    let dataList = [];

    dataBaseList.forEach( info => {
        let changeData:any = extractData(resultConfig.FuHuaQiBaseListConfig, info, true);
        // let hatchingGround = info.hatchingGround;
        // hatchingGround.forEach (info => {
        //     let {personInChargeAdd, siteAcreage, leasePrice} = info;
        //     changeData.personInChargeAdd = personInChargeAdd;
        //     changeData.siteAcreage = siteAcreage;
        //     changeData.leasePrice = leasePrice;
        // })
        let enterpriseTotal = enterpriseMap[info.uscc] || 0;
        let numOfEntities = enterpriseTotal - (info.virtualEnterpriseNum || 0 );
        numOfEntities = numOfEntities < 0 ? 0 : numOfEntities;

        changeData.enterpriseTotal = enterpriseTotal;
        changeData.numOfEntities = numOfEntities;

        changeData.lv = changeEnumValue(configEnum.FUHUAQILV, changeData.lv);
        changeData.industry = changeEnumValue(configEnum.FUHUAINDUSTRY, changeData.industry);
        changeData.institutionalNature = changeEnumValue(configEnum.INSTITUTIONALNATURE, changeData.institutionalNature);
        changeData.operationModel = changeEnumValue(configEnum.OPERATIONMODEL, changeData.operationModel);
        if (changeData.operationModel == "_投资__孵化_类型"){
            changeData.operationModel = changeData.operationModel.replace("__","+");
            changeData.operationModel = changeData.operationModel.replace("_","“");
            changeData.operationModel = changeData.operationModel.replace("_","”");
        }

        changeData.foundingTeamStr = "无";
        let foundingTeam = [];
        if (changeData.foundingTeam && changeData.foundingTeam.length) {
            changeData.foundingTeam.forEach(item => {
                let {memberName,  memberAge,  memberSchool,  des} = item;
                let type = changeEnumValue(configEnum.TEAM, item.type);//人才类型
                foundingTeam.push({
                    memberName,  memberAge,  memberSchool,  des, type,
                    memberSex : changeEnumValue(configEnum.SEX, item.memberSex),//成员性别
                    memberDEGREE : changeEnumValue(configEnum.DEGREE, item.memberDEGREE),//成员最高学历
                });
                // item.memberDEGREE = changeEnumValue(configEnum.DEGREE, item.foundingTeamStr);//成员最高学历
                /* 这里兼容了一下，团队类型数据库里存到了每一个人的信息里 以防后续要修改 */
                changeData.foundingTeamStr = type;
            });
            changeData.foundingTeam = foundingTeam;
        }

        dataList.push(changeData);
    });

    return {count, dataList};
}



/**
 * 导出符合条件的孵化器基本信息数据 
 * @param lv 
 * @param institutionalNature 
 * @param industry 
 * @returns 
 */
export async function outPutFuHuaQiBaseData(lv:number, institutionalNature:number, industry:number, type:Number) {
    let selectParam:any = {};

    if (type == configEnum.OUTPUTTYPE.当前数据) {
        if (lv) selectParam.lv = lv;
        if (institutionalNature) selectParam.institutionalNature = institutionalNature;
        if (industry) selectParam.industry = industry;
    }

    let enterpriseMap = await groupFindEnterprise();//企业总数

    let dataBaseList = await fuhuaqiData.findFuHuaQiList(selectParam);
    let dataList = [];

    dataBaseList.forEach( info => {
        let changeData:any = extractData(resultConfig.FuHuaQiBaseListConfig, info, true);

        let enterpriseTotal = enterpriseMap[info.uscc] || 0;
        let numOfEntities = enterpriseTotal - (info.virtualEnterpriseNum || 0 );
        numOfEntities = numOfEntities < 0 ? 0 : numOfEntities;

        changeData.enterpriseTotal = enterpriseTotal;
        changeData.numOfEntities = numOfEntities;

        changeData.lv = changeEnumValue(configEnum.FUHUAQILV, changeData.lv);
        changeData.industry = changeEnumValue(configEnum.FUHUAINDUSTRY, changeData.industry);
        changeData.institutionalNature = changeEnumValue(configEnum.INSTITUTIONALNATURE, changeData.institutionalNature);
        changeData.operationModel = changeEnumValue(configEnum.OPERATIONMODEL, changeData.operationModel);
        if (changeData.operationModel == "_投资__孵化_类型"){
            changeData.operationModel = changeData.operationModel.replace("__","+");
            changeData.operationModel = changeData.operationModel.replace("_","“");
            changeData.operationModel = changeData.operationModel.replace("_","”");
        }

        changeData.foundingTeamStr = "无";
        let foundingTeam = [];
        if (changeData.foundingTeam && changeData.foundingTeam.length) {
            changeData.foundingTeam.forEach(item => {
                let {memberName,  memberAge,  memberSchool,  des} = item;
                let type = changeEnumValue(configEnum.TEAM, item.type);//人才类型
                foundingTeam.push({
                    memberName,  memberAge,  memberSchool,  des, type,
                    memberSex : changeEnumValue(configEnum.SEX, item.memberSex),//成员性别
                    memberDEGREE : changeEnumValue(configEnum.DEGREE, item.memberDEGREE),//成员最高学历
                });
                // item.memberDEGREE = changeEnumValue(configEnum.DEGREE, item.foundingTeamStr);//成员最高学历
                /* 这里兼容了一下，团队类型数据库里存到了每一个人的信息里 以防后续要修改 */
                changeData.foundingTeamStr = type;
            });
            changeData.foundingTeam = foundingTeam;
        }

        dataList.push(changeData);
    });

    
    let keyList = [
        "name","operationName","uscc","virtualEnterpriseNum","logonTime",
        "incubatedAcreage", "acreageTotal", "acreagePersonalUse", "lv", "identificationTime",
        "industry", "institutionalNature", "liaison", "liaisonPhone", "personInCharge",
        "personInChargePhone", "personInChargeAdd", "siteAcreage", "leasePrice"
    ];
    let titleList = [
        "名称", "运营机构名称", "统一信用代码", "虚拟企业数量", "注册时间",
        "在孵面积", "孵化器总面积", "孵化器自用面积", "孵化器级别", "认定时间",
        "领域", "机构性质", "联系人", "联系电话", "负责人",
        "负责人联系电话", "孵化场地地址", "孵化场地面积", "孵化场地出租单价"
    ];
    let result = [titleList];
    dataList.forEach(info => {
        let subList = [];
        keyList.forEach(subInfo => {
            subList.push(info[subInfo] || '');
        });
        result.push(subList);
    });

    return result;
}